==> soliq_id-1.4.27.tar.gz: lib/src/utils/camera_view.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:ui';

import 'package:camera/camera.dart';
import 'package:google_mlkit_face_detection/google_mlkit_face_detection.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:lottie/lottie.dart';
import 'package:soliq_id/src/utils/progress_indicator/src/circular_step_progress_indicator.dart';
import 'package:soliq_id/src/utils/utils.dart';
import 'dart:math' as math;
import '../../soliq_id.dart';
import 'color_utils.dart';

class CameraView extends StatefulWidget {
  const CameraView({
    Key? key,
    required this.title,
    required this.customPaint,
    this.text,
    required this.onImage,
    required this.onFinish,
    this.initialDirection = CameraLensDirection.back,
    required this.isValid,
    this.value = 0,
    this.suggestAngle,
  }) : super(key: key);

  final String title;
  final CustomPaint? customPaint;
  final String? text;
  final Function(InputImage inputImage, String imageString) onImage;
  final Function(String filepath) onFinish;
  final CameraLensDirection initialDirection;
  final bool isValid;
  final double value;
  final double? suggestAngle;

  @override
  State<CameraView> createState() => _CameraViewState();
}

class _CameraViewState extends State<CameraView> {
  CameraController? _controller;
  int _cameraIndex = -1;
  double zoomLevel = 0.0, minZoomLevel = 0.0, maxZoomLevel = 0.0;
  var lastCheckTime = DateTime.now();

  @override
  void initState() {
    super.initState();
    if (cameras.any(
      (element) =>
          element.lensDirection == widget.initialDirection &&
          element.sensorOrientation == 90,
    )) {
      _cameraIndex = cameras.indexOf(
        cameras.firstWhere((element) =>
            element.lensDirection == widget.initialDirection &&
            element.sensorOrientation == 90),
      );
    } else {
      for (var i = 0; i < cameras.length; i++) {
        if (cameras[i].lensDirection == widget.initialDirection) {
          _cameraIndex = i;
          break;
        }
      }
    }

    if (_cameraIndex != -1) {
      _startLiveFeed();
    }
  }

  @override
  void dispose() {
    _stopLiveFeed();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _liveFeedBody();
  }

  Widget _liveFeedBody() {
    const double previewAspectRatio = 1;
    if (_controller?.value.isInitialized == false) {
      return Container();
    }

    return Container(
        alignment: Alignment.topCenter,
        margin: const EdgeInsets.only(
          left: 16,
          right: 16,
        ),
        child:
            // widget.isValid ?
            AspectRatio(
                aspectRatio: 1.0 / previewAspectRatio,
                child: CircularStepProgressIndicator(
                  totalSteps: 100,
                  currentStep: widget.value,
                  stepSize: 20,
                  selectedColor: AppColor.chartActiveColor,
                  unselectedColor: Colors.grey.withOpacity(0.2),
                  padding: math.pi / 35,
                  child: Stack(
                    alignment: Alignment.center,
                    children: [
                      Container(
                        margin: const EdgeInsets.all(8),
                        child: ClipRRect(
                          borderRadius: BorderRadius.all(Radius.circular(
                              MediaQuery.of(context).size.width / 2)),
                          child: Transform.scale(
                            scale: _controller!.value.aspectRatio /
                                previewAspectRatio,
                            child: Stack(
                              children: [
                                Center(
                                  child: CameraPreview(_controller!),
                                ),
                                Visibility(
                                  visible: widget.value >= 96,
                                  child: Stack(
                                    children: [
                                      BackdropFilter(
                                        filter: ImageFilter.blur(
                                            sigmaX: 5, sigmaY: 5),
                                        child: const SizedBox(),
                                      ),
                                      const Align(
                                          alignment: Alignment.center,
                                          child: CupertinoActivityIndicator(
                                            radius: 20,
                                            color: Colors.white,
                                          )),
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                        ),
                      ),
                      if (widget.suggestAngle != null)
                        Container(
                          margin: const EdgeInsets.all(8),
                          child: BackdropFilter(
                            filter: ImageFilter.blur(sigmaX: 3, sigmaY: 3),
                            child: ClipRRect(
                              borderRadius: BorderRadius.all(Radius.circular(
                                  MediaQuery.of(context).size.width / 2)),
                              child: Transform.scale(
                                scale: _controller!.value.aspectRatio /
                                    previewAspectRatio,
                                child: Center(
                                  child: Transform.rotate(
                                      angle: widget.suggestAngle ?? -math.pi / 2,
                                      child: Lottie.asset(
                                          "packages/soliq_id/lib/src/assets/lottie/arrow.json")),
                                ),
                              ),
                            ),
                          ),
                        ),
                    ],
                  ),
                ))

        );
  }

  Future _startLiveFeed() async {
    final camera = cameras[_cameraIndex];
    _controller = CameraController(
      camera,
      ResolutionPreset.high,
      enableAudio: false,
    );

    _controller?.initialize().then((_) {
      if (!mounted) {
        return;
      }
      _controller?.getMinZoomLevel().then((value) {
        zoomLevel = value;
        minZoomLevel = value;
      });
      _controller?.getMaxZoomLevel().then((value) {
        maxZoomLevel = value;
      });
      _controller?.startImageStream(_processCameraImage);
      setState(() {});
    });
  }

  Future _stopLiveFeed() async {
    if (!(widget.value >= 96)) await _controller?.stopImageStream();
    await _controller?.dispose();
    _controller = null;
  }

  Future _processCameraImage(CameraImage image) async {
    if (DateTime.now().millisecondsSinceEpoch -
            lastCheckTime.millisecondsSinceEpoch <
        150) {
      return;
    }
    lastCheckTime = DateTime.now();
    // print("object1");
    final WriteBuffer allBytes = WriteBuffer();
    for (final Plane plane in image.planes) {
      allBytes.putUint8List(plane.bytes);
    }
    final bytes = allBytes.done().buffer;

    final Size imageSize =
        Size(image.width.toDouble(), image.height.toDouble());

    final camera = cameras[_cameraIndex];
    final imageRotation =
        InputImageRotationValue.fromRawValue(camera.sensorOrientation);
    if (imageRotation == null) return;

    final inputImageFormat =
        InputImageFormatValue.fromRawValue(image.format.raw);
    if (inputImageFormat == null) return;

    final planeData = image.planes.map(
      (Plane plane) {
        return InputImagePlaneMetadata(
          bytesPerRow: plane.bytesPerRow,
          height: plane.height,
          width: plane.width,
        );
      },
    ).toList();

    final inputImageData = InputImageData(
      size: imageSize,
      imageRotation: imageRotation,
      inputImageFormat: inputImageFormat,
      planeData: planeData,
    );

    final inputImage = InputImage.fromBytes(
        bytes: bytes.asUint8List(), inputImageData: inputImageData);

    String? filepath;
    if (widget.value >= 96) {
      if (_controller != null) {
        await _controller?.stopImageStream();
        var xFile = await _controller!.takePicture().catchError((onError){
          Utils.onError(context, onError);
        }).whenComplete(() => log("onFinish: complete"));
        filepath = xFile.path;

        Timer.periodic(const Duration(seconds: 1), (timer) {

          if(timer.tick == 2){
            timer.cancel();
            _controller?.pausePreview();
            _controller?.dispose();
          }
        });
      }
    }

    widget.onImage(inputImage, base64.encode(Uint8List.view(bytes)));
    try{
      log("onFinish: $filepath");
      widget.onFinish(filepath ?? "");
    } catch(e){
      log("onFinish: $e");
    }
  }
}

==> soliq_id-1.4.36.tar.gz: lib/src/utils/camera_view.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:ui';

import 'package:camera/camera.dart';
import 'package:google_mlkit_face_detection/google_mlkit_face_detection.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:lottie/lottie.dart';
import 'package:soliq_id/src/utils/progress_indicator/src/circular_step_progress_indicator.dart';
import 'package:soliq_id/src/utils/utils.dart';
import 'dart:math' as math;
import '../../soliq_id.dart';
import 'color_utils.dart';

class CameraView extends StatefulWidget {
  const CameraView({
    Key? key,
    required this.title,
    required this.customPaint,
    this.text,
    required this.onImage,
    required this.onFinish,
    this.initialDirection = CameraLensDirection.back,
    required this.isValid,
    this.value = 0,
    this.suggestAngle,
  }) : super(key: key);

  final String title;
  final CustomPaint? customPaint;
  final String? text;
  final Function(InputImage inputImage, String imageString) onImage;
  final Function(String filepath) onFinish;
  final CameraLensDirection initialDirection;
  final bool isValid;
  final double value;
  final double? suggestAngle;

  @override
  State<CameraView> createState() => _CameraViewState();
}

class _CameraViewState extends State<CameraView> {
  CameraController? _controller;
  int _cameraIndex = -1;
  double zoomLevel = 0.0, minZoomLevel = 0.0, maxZoomLevel = 0.0;
  var lastCheckTime = DateTime.now();

  @override
  void initState() {
    super.initState();
    if (cameras.any(
      (element) =>
          element.lensDirection == widget.initialDirection &&
          element.sensorOrientation == 90,
    )) {
      _cameraIndex = cameras.indexOf(
        cameras.firstWhere((element) =>
            element.lensDirection == widget.initialDirection &&
            element.sensorOrientation == 90),
      );
    } else {
      for (var i = 0; i < cameras.length; i++) {
        if (cameras[i].lensDirection == widget.initialDirection) {
          _cameraIndex = i;
          break;
        }
      }
    }

    if (_cameraIndex != -1) {
      _startLiveFeed();
    }
  }

  @override
  void dispose() {
    _stopLiveFeed();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _liveFeedBody();
  }

  Widget _liveFeedBody() {
    const double previewAspectRatio = 1;
    if (_controller?.value.isInitialized == false) {
      return Container();
    }

    return Container(
        alignment: Alignment.topCenter,
        margin: const EdgeInsets.only(
          left: 16,
          right: 16,
        ),
        child:
            // widget.isValid ?
            AspectRatio(
                aspectRatio: 1.0 / previewAspectRatio,
                child: CircularStepProgressIndicator(
                  totalSteps: 100,
                  currentStep: widget.value,
                  stepSize: 20,
                  selectedColor: AppColor.chartActiveColor,
                  unselectedColor: Colors.grey.withOpacity(0.2),
                  padding: math.pi / 35,
                  child: Stack(
                    alignment: Alignment.center,
                    children: [
                      Container(
                        margin: const EdgeInsets.all(8),
                        child: ClipRRect(
                          borderRadius: BorderRadius.all(Radius.circular(
                              MediaQuery.of(context).size.width / 2)),
                          child: Transform.scale(
                            scale: _controller!.value.aspectRatio /
                                previewAspectRatio,
                            child: Stack(
                              children: [
                                Center(
                                  child: CameraPreview(_controller!),
                                ),
                                Visibility(
                                  visible: widget.value >= 96,
                                  child: Stack(
                                    children: [
                                      BackdropFilter(
                                        filter: ImageFilter.blur(
                                            sigmaX: 5, sigmaY: 5),
                                        child: const SizedBox(),
                                      ),
                                      const Align(
                                          alignment: Alignment.center,
                                          child: CupertinoActivityIndicator(
                                            radius: 20,
                                            color: Colors.white,
                                          )),
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                        ),
                      ),
                      if (widget.suggestAngle != null)
                        Container(
                          margin: const EdgeInsets.all(8),
                          child: BackdropFilter(
                            filter: ImageFilter.blur(sigmaX: 3, sigmaY: 3),
                            child: ClipRRect(
                              borderRadius: BorderRadius.all(Radius.circular(
                                  MediaQuery.of(context).size.width / 2)),
                              child: Transform.scale(
                                scale: _controller!.value.aspectRatio /
                                    previewAspectRatio,
                                child: Center(
                                  child: Transform.rotate(
                                      angle:
                                          widget.suggestAngle ?? -math.pi / 2,
                                      child: Lottie.asset(
                                          "packages/soliq_id/lib/src/assets/lottie/arrow.json")),
                                ),
                              ),
                            ),
                          ),
                        ),
                    ],
                  ),
                )));
  }

  Future _startLiveFeed() async {
    final camera = cameras[_cameraIndex];
    _controller = CameraController(
      camera,
      ResolutionPreset.high,
      enableAudio: false,
    );

    _controller?.initialize().then((_) {
      if (!mounted) {
        return;
      }
      _controller?.getMinZoomLevel().then((value) {
        zoomLevel = value;
        minZoomLevel = value;
      });
      _controller?.getMaxZoomLevel().then((value) {
        maxZoomLevel = value;
      });
      _controller?.startImageStream(_processCameraImage);
      setState(() {});
    });
  }

  Future _stopLiveFeed() async {
    if (!(widget.value >= 96)) await _controller?.stopImageStream();
    await _controller?.dispose();
    _controller = null;
  }

  Future _processCameraImage(CameraImage image) async {
    if (DateTime.now().millisecondsSinceEpoch -
            lastCheckTime.millisecondsSinceEpoch <
        150) {
      return;
    }
    lastCheckTime = DateTime.now();
    // print("object1");
    final WriteBuffer allBytes = WriteBuffer();
    for (final Plane plane in image.planes) {
      allBytes.putUint8List(plane.bytes);
    }
    final bytes = allBytes.done().buffer;

    final Size imageSize =
        Size(image.width.toDouble(), image.height.toDouble());

    final camera = cameras[_cameraIndex];
    final imageRotation =
        InputImageRotationValue.fromRawValue(camera.sensorOrientation);
    if (imageRotation == null) return;

    final inputImageFormat =
        InputImageFormatValue.fromRawValue(image.format.raw);
    if (inputImageFormat == null) return;

    // final planeData = image.planes.map(
    //   (Plane plane) {
    //     return InputImagePlaneMetadata(
    //       bytesPerRow: plane.bytesPerRow,
    //       height: plane.height,
    //       width: plane.width,
    //     );
    //   },
    // ).toList();

    // final inputImageData = InputImageData(
    //   size: imageSize,
    //   imageRotation: imageRotation,
    //   inputImageFormat: inputImageFormat,
    //   planeData: planeData,
    // );
    if (image.planes.length != 1) return null;
    final plane = image.planes.first;

    final inputImage = InputImage.fromBytes(
        bytes: bytes.asUint8List(),
        metadata: InputImageMetadata(
            size: Size(image.width.toDouble(), image.height.toDouble()),
            rotation: imageRotation,
            format: inputImageFormat,
            bytesPerRow: plane.bytesPerRow));

    String? filepath;
    if (widget.value >= 96) {
      if (_controller != null) {
        await _controller?.stopImageStream();
        var xFile = await _controller!.takePicture().catchError((onError) {
          Utils.onError(context, onError);
        }).whenComplete(() => log("onFinish: complete"));
        filepath = xFile.path;

        Timer.periodic(const Duration(seconds: 1), (timer) {
          if (timer.tick == 2) {
            timer.cancel();
            _controller?.pausePreview();
            _controller?.dispose();
          }
        });
      }
    }

    widget.onImage(inputImage, base64.encode(Uint8List.view(bytes)));
    try {
      log("onFinish: $filepath");
      widget.onFinish(filepath ?? "");
    } catch (e) {
      log("onFinish: $e");
    }
  }
}
